!{\src2tex{textfont=tt}}
!!****f* USPP2PAW/calc_vhtnzc
!! NAME
!! calc_vhtnzc
!!
!! FUNCTION
!! Compute vh(tnZc) ) and the <phi_i|vh(tnZc)|phi_j> matrix elements
!! Have to descreen it
!!
!! COPYRIGHT
!! Copyright (C) 1998-2014 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the GNU General Public License
!! See http://www.gnu.org/copyleft/gpl.txt
!!
!! INPUTS
!!  option=0 if "bare" potential is forced to zero
!!         1 if "bare" potential is computed
!!  pawps
!!    %coreden4pr2(max_meshsz)= Core density multiplied by 4Pi.r2
!!    %rvloc0(max_meshsz)= Local (descreened) potential * r
!!    %shapefunc(wav_meshsz)= Normalized shape function
!!    %tcoreden4pr2(max_meshsz)= Pseudo core density multiplied by 4Pi.r2
!!  pawrad=datastructure containing mesh size definitions
!!  pshead
!!    %atomic_charge= Total atomic charge
!!    %max_meshsz=Maximal mesh size for densities/potentials
!!    %valence_charge= Valence charge
!!    %vbar_meshsz= Dimension of radial mesh for Vbare
!!    %vloc_meshsz= Dimension of radial mesh for vloc=vhtnzc
!!    %wav_meshsz=Mesh size for partial waves and projectors inherited from uspp
!!  un_log= Unit number for log file (comments)
!!
!! OUTPUT
!!  pawps
!!    %vbare(wav_meshsz)= Vbare local potential used in Blochl/Holzwarth's formulation
!!    %vhtnzc(max_meshsz)= Hartree potential of the ps-density
!!                         of the nucleus + core electrons
!!
!! NOTES
!! When writing Vbare (in wrpawps), do not forget to write it up to RcLoc !
!!
!! PARENTS
!!      uspp2paw
!!
!! CHILDREN
!!      csimp,deducer0,poisson
!!
!! SOURCE

 subroutine calc_vhtnzc(exfact,pawrad,pawps,pshead,option,un_log,usexcnhat)


  use defs_basis
  use defs_pawps

  implicit none

!Arguments ---------------------------------------------
 integer :: option,un_log,usexcnhat
 real(dp) :: exfact
 type(pawrad_type)   :: pawrad
 type(pawps_type)    :: pawps
 type(pshead_type)   :: pshead

!Local variables ---------------------------------------
 integer :: meshsz,maxmsz,vbrmsz,ib
 real(dp) :: qcore,rmax_vloc
 real(dp),allocatable :: den(:),ff(:),rvbare(:),rvhtnzc1(:),den4pr2(:),hatden4pr2(:),rvxc(:),rvxc1(:)
!--------------------------------------------------------
 
 meshsz=pshead%wav_meshsz
 maxmsz=pshead%max_meshsz

!Compute Qcore
 allocate(ff(pshead%max_meshsz));ff(:)=pawps%coreden4pr2(:)-pawps%tcoreden4pr2(:)
 call csimp(ff,pawrad,pshead%max_meshsz,qcore)
 deallocate(ff)

 allocate(rvbare(maxmsz),rvhtnzc1(maxmsz))

!Compute r*Vh(tnZC) with Poisson equation and vbare=0
!----------------------------------------------------
 allocate(den(maxmsz))
 den(1:maxmsz)=pawps%tcoreden4pr2(1:maxmsz)
 den(1:meshsz)=den(1:meshsz)+(qcore-pshead%atomic_charge)*pawps%shapefunc(1:meshsz)*pawrad%rad(1:meshsz)**2
 call poisson(den,maxmsz,pawrad,rvhtnzc1)
 deallocate(den)

!Compute vbare inside sphere by difference
!-----------------------------------------
!Eventually impose vbare=0 inside sphere (for test purpose only)
 allocate(rvxc(maxmsz),rvxc1(maxmsz))
 rvbare=zero
 if (option/=0) then
   if (usexcnhat==0) rvbare(:)=pawps%rvloc0(:)-rvhtnzc1(:)
   if (usexcnhat==1)then
      allocate(den4pr2(maxmsz),hatden4pr2(meshsz))
      hatden4pr2(1:meshsz)=pawps%shpnrm(0)*pawps%shapefunc(1:meshsz)*pawrad%rad(1:meshsz)**2
      den4pr2(1:maxmsz)=pawps%tvaleden4pr2(1:maxmsz)+pawps%tcoreden4pr2(1:maxmsz)
      call calc_vxc(den4pr2,maxmsz,pawrad,rvxc,exfact,un_log)
      den4pr2(1:meshsz)=den4pr2(1:meshsz)+hatden4pr2(1:meshsz)
      call calc_vxc(den4pr2,maxmsz,pawrad,rvxc1,exfact,un_log)
      rvbare(:)=pawps%rvloc0(:)-rvhtnzc1(:)+rvxc1(:)-rvxc(:)
      deallocate (den4pr2,hatden4pr2)
   end if
 endif

!Compute Vh(tnZC) everywhere, imposing vbare=0 outside sphere
!--------------------------------------------------------------
 vbrmsz=max(meshsz,pshead%vbar_meshsz)
! vbrmsz=pshead%vbar_meshsz+5

 pawps%vhtnzc(:)=rvhtnzc1(:)
 if (usexcnhat==0) pawps%vhtnzc(1:vbrmsz)=pawps%vhtnzc(1:vbrmsz)+rvbare(1:vbrmsz)
 if (usexcnhat==1) pawps%vhtnzc(1:vbrmsz)=pawps%vhtnzc(1:vbrmsz)+rvbare(1:vbrmsz)-rvxc1(1:vbrmsz)+rvxc(1:vbrmsz)
 pawps%vhtnzc(2:maxmsz)=pawps%vhtnzc(2:maxmsz)/pawrad%rad(2:maxmsz)
 call deducer0(pawps%vhtnzc,maxmsz,pawrad)
 pawps%vbare(1:vbrmsz)=rvbare(1:vbrmsz)
 pawps%vbare(2:maxmsz)=pawps%vbare(2:maxmsz)/pawrad%rad(2:maxmsz)
 call deducer0(pawps%vbare,maxmsz,pawrad)

 deallocate(rvbare,rvhtnzc1,rvxc,rvxc1)


!MODIF
!Put Vh(tnZC) to -Z/r above 10 a.u.
 rmax_vloc=10.d0
 maxmsz=int(log(1.d0+rmax_vloc/pshead%rad_step)/pshead%log_step)+1
 pawps%vhtnzc(maxmsz:pshead%vloc_meshsz)=-pshead%valence_charge/pawrad%rad(maxmsz:pshead%vloc_meshsz)
!MODIF

!Test the behaviour of VhtnZc at long range: vhtnzc(r)=-Z/r ?
!-----------------------------------------------------------
  write(un_log,'(/,a)') '> USPP2PAW INFO:'
  write(un_log,'(a,f9.2,a,a,es11.4)') &
&   '  At r=',rmax_vloc,' a.u.,',&
&   ' VHartree(ntild(Zv+Zc))= -Zv/r + ',&
&     pawps%vhtnzc(maxmsz-1)+pshead%valence_charge/pawrad%rad(maxmsz-1)
  write(un_log,'(a)') &
&   '  This quantity must be as small as possible.'

 end subroutine calc_vhtnzc

!!***
